package helper;

public class BloodTypeServices {

	/**
	 * Determines whether the blood type of the child is consistent genetically
	 * with the blood types of the mother and father.
	 * 
	 * http://www.canadiancrc.com/paternity_determination_blood_type.aspx
	 * http://mistupid.com/health/bloodinherit.htm
	 * 
	 * @param motherBloodType
	 *            a string representing a blood type, which must be one of A, B,
	 *            AB, or O (case insenstive)
	 * @param fatherBloodType
	 *            a string representing a blood type, which must be one of A, B,
	 *            AB, or O (case insenstive)
	 * @param childBloodType
	 *            a string representing a blood type, which must be one of A, B,
	 *            AB, or O (case insenstive)
	 * @return a boolean value that is true if and only if the blood type of the
	 *         child is consistent genetically with the blood types of the
	 *         mother and father
	 * @throws RuntimeException
	 *             if any of the passed parameters are not valid blood types,
	 *             according to the specification above. The message of the
	 *             thrown exception will be "Invalid blood type!".
	 */
	public static boolean isCompatible(String motherBloodType,
			String fatherBloodType, String childBloodType)
			throws RuntimeException {
		boolean isConsistent = false;
		motherBloodType = motherBloodType.toUpperCase();
		fatherBloodType = fatherBloodType.toUpperCase();
		childBloodType = childBloodType.toUpperCase();
		boolean isOKm = motherBloodType.matches("[ABO]")
				|| motherBloodType.matches("AB");
		boolean isOKf = fatherBloodType.matches("[ABO]")
				|| fatherBloodType.matches("AB");
		boolean isOKc = childBloodType.matches("[ABO]")
				|| childBloodType.matches("AB");
		if (!(isOKm || isOKf || isOKc)) {
			throw new RuntimeException("Invalid blood type!");
		}

		if (motherBloodType.equals("A")) {
			if (fatherBloodType.equals("A")) {
				if (childBloodType.equals("A")) {
					isConsistent = true;
				} else if (childBloodType.equals("O")) {
					isConsistent = true;
				}
			} else if (fatherBloodType.equals("B")) {
				// any child blood type is possible
				isConsistent = true;
			} else if (fatherBloodType.equals("AB")) {
				// any child blood type except O is possible
				isConsistent = true;
				if (childBloodType.equals("O")) {
					isConsistent = false;
				}
			} else if (fatherBloodType.equals("O")) {
				if (childBloodType.equals("A")) {
					isConsistent = true;
				} else if (childBloodType.equals("O")) {
					isConsistent = true;
				}
			}
		} else if (motherBloodType.equals("B")) {
			if (fatherBloodType.equals("A")) {
				// any child blood type is possible
				isConsistent = true;
			} else if (fatherBloodType.equals("B")) {
				if (childBloodType.equals("B")) {
					isConsistent = true;
				}
				if (childBloodType.equals("O")) {
					isConsistent = true;
				}
			} else if (fatherBloodType.equals("AB")) {
				// any child blood type except O is possible
				isConsistent = true;
				if (childBloodType.equals("O")) {
					isConsistent = false;
				}
			} else if (fatherBloodType.equals("O")) {
				if (childBloodType.equals("B")) {
					isConsistent = true;
				}
				if (childBloodType.equals("O")) {
					isConsistent = true;
				}
			}
		} else if (motherBloodType.equals("AB")) {
			if (fatherBloodType.equals("A")) {
				// any child blood type is possible except O
				isConsistent = true;
				if (childBloodType.equals("O")) {
					isConsistent = false;
				}
			} else if (fatherBloodType.equals("B")) {
				// any child blood type is possible except O
				isConsistent = true;
				if (childBloodType.equals("O")) {
					isConsistent = false;
				}
			} else if (fatherBloodType.equals("AB")) {
				// any child blood type is possible except O
				isConsistent = true;
				if (childBloodType.equals("O")) {
					isConsistent = false;
				}
			} else if (fatherBloodType.equals("O")) {
				if (childBloodType.equals("B")) {
					isConsistent = true;
				}
				if (childBloodType.equals("A")) {
					isConsistent = true;
				}
			}
		} else if (motherBloodType.equals("O")) {
			if (fatherBloodType.equals("A")) {
				if (childBloodType.equals("A")) {
					isConsistent = true;
				}
				if (childBloodType.equals("O")) {
					isConsistent = true;
				}
			} else if (fatherBloodType.equals("B")) {
				if (childBloodType.equals("B")) {
					isConsistent = true;
				}
				if (childBloodType.equals("O")) {
					isConsistent = true;
				}
			} else if (fatherBloodType.equals("AB")) {
				if (childBloodType.equals("A")) {
					isConsistent = true;
				}
				if (childBloodType.equals("B")) {
					isConsistent = true;
				}
			} else if (fatherBloodType.equals("O")) {
				if (childBloodType.equals("O")) {
					isConsistent = true;
				}
			}
		}
		return isConsistent;
	}
}
